@@ -7,7 +7,7 @@ async fn main() {
   let duration = 60;
   let r = daemon("rmw upnp test".to_string(), port, duration);
   loop {
-    println!("{:?}", r.recv().unwrap());
+    println!("{:?}", r.recv().await);
     exit(0);
   }
 }
