@@ -1,14 +1,16 @@
 use anyhow::Result;
-use async_std::task::{sleep, spawn};
+use async_std::{
+  channel::{bounded, Receiver},
+  task::{sleep, spawn},
+};
 use igd::aio::search_gateway;
 use igd::AddPortError::{self, PortInUse};
 use std::net::TcpStream;
 use std::net::{IpAddr, Ipv4Addr, SocketAddrV4};
-use std::sync::mpsc::{channel, Receiver};
 use std::time::Duration;
 
 #[derive(thiserror::Error, Debug)]
-pub enum Error {
+pub enum UpnpError {
   #[error("not ipv4")]
   NotIpv4,
 }
@@ -17,10 +19,14 @@ pub fn daemon(
   name: String,
   port: u16,
   duration: u32,
-) -> Receiver<Result<(u16, Ipv4Addr, SocketAddrV4), String>> {
-  let (s, r) = channel();
+) -> Receiver<Result<(u16, Ipv4Addr, SocketAddrV4)>> {
+  let (s, r) = bounded(1);
 
   spawn(async move {
+    if false {
+      return Ok::<(), anyhow::Error>(());
+    };
+
     let mut local_ip = Ipv4Addr::UNSPECIFIED;
     let mut pre_gateway = SocketAddrV4::new(local_ip, 0);
     let duration = duration + 9;
@@ -34,25 +40,27 @@ pub fn daemon(
             retryed = 0;
             local_ip = ip;
             pre_gateway = gateway;
-            let _ = s.send(Ok((ext_port, ip, gateway)));
+            s.send(Ok((ext_port, ip, gateway))).await?;
           }
         }
-        Err(err) => {
+        Err(error) => {
           local_ip = Ipv4Addr::UNSPECIFIED;
-          let err = err.root_cause();
-          if retryed < 9 {
-            if let Some(PortInUse) = err.downcast_ref::<AddPortError>() {
-              retryed += 1;
-              if ext_port == 65535 {
-                ext_port = 1025;
-              } else {
-                ext_port += 1;
+          {
+            let err = error.root_cause();
+            if retryed < 9 {
+              if let Some(PortInUse) = err.downcast_ref::<AddPortError>() {
+                retryed += 1;
+                if ext_port == 65535 {
+                  ext_port = 1025;
+                } else {
+                  ext_port += 1;
+                }
+                continue;
               }
-              continue;
             }
           }
           retryed = 0;
-          let _ = s.send(Err(err.to_string()));
+          s.send(Err(error)).await?;
         }
       }
       sleep(seconds).await;
@@ -84,6 +92,6 @@ pub async fn upnp(
       .await?;
     Ok((gateway_addr, ip))
   } else {
-    Err(Error::NotIpv4.into())
+    Err(UpnpError::NotIpv4.into())
   }
 }
